package com.typowebguide.directory;

import java.io.Serializable;
import java.util.NavigableMap;
import java.util.NavigableSet;
import java.util.TreeMap;
import java.util.TreeSet;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;

@Entity
public class Entry implements Comparable<Entry>, Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	@Id protected long id;
	private String name;
	private String description;
	@ManyToOne(cascade=CascadeType.ALL) private Folder parent;
	private boolean listable;
	@ManyToMany(cascade=CascadeType.ALL) private NavigableMap<Folder, String> shadowParents = new TreeMap<Folder, String>();
	
	@Override
	public int compareTo(Entry that) {
		return this.getName().compareTo(that.getName());
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((getDescription() == null) ? 0 : getDescription().hashCode());
		result = prime * result + ((getName() == null) ? 0 : getName().hashCode());
		result = prime * result + ((getParent() == null) ? 0 : getParent().hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Entry other = (Entry) obj;
		if (getDescription() == null) {
			if (other.getDescription() != null)
				return false;
		} else if (!getDescription().equals(other.getDescription()))
			return false;
		if (getName() == null) {
			if (other.getName() != null)
				return false;
		} else if (!getName().equals(other.getName()))
			return false;
		if (getParent() == null) {
			if (other.getParent() != null)
				return false;
		} else if (!getParent().equals(other.getParent()))
			return false;
		return true;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Folder getParent() {
		return parent;
	}

	public void setParent(Folder parent) {
		this.parent = parent;
	}
	
	public NavigableSet<Folder> getShadowParents() {
		return new TreeSet<Folder>(shadowParents.navigableKeySet());
	}

	public void setListable(boolean listable) {
		this.listable = listable;
	}

	public boolean isListable() {
		return listable && isShown();
	}
	
	public boolean isShown() {
		return true;
	}

	public void addShadowParent(Folder f) {
		addShadowParent(f, f.getName());
	}
	
	public void addShadowParent(Folder f, String name) {
		f.addChild(this);
	}
	
	public void removeShadowParent(Folder f) {
		getShadowParents().remove(f);
		f.removeChild(this);
	}
		
	@Override
	public String toString() {
		return (this.getParent()==null ? "" : this.getParent().toString()) + "/" + getName();
	}

	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

}
